<?php

namespace Drupal\acquia_search;

use Drupal\acquia_connector\Subscription;
use Drupal\Component\Datetime\TimeInterface;
use Drupal\Component\Serialization\Json;
use Drupal\Component\Utility\Crypt;
use Drupal\Core\Cache\CacheBackendInterface;
use Drupal\Core\Logger\LoggerChannelInterface;
use GuzzleHttp\Client;
use GuzzleHttp\Exception\RequestException;

/**
 * Acquia Search API Client.
 *
 * Not to be confused with Search API, this is the service that works with RAGE
 * to fetch the cores available to a customer.
 *
 * @package Drupal\acquia_search
 */
class AcquiaSearchApiClient {

  /**
   * Cache backend.
   *
   * @var \Drupal\Core\Cache\CacheBackendInterface
   */
  protected $cache;

  /**
   * The HTTP client to fetch the feed data with.
   *
   * @var \GuzzleHttp\Client
   */
  protected $client;

  /**
   * Acquia Search Logger Channel.
   *
   * @var \Drupal\Core\Logger\LoggerChannelInterface
   */
  protected $logger;

  /**
   * HTTP headers.
   *
   * @var array
   */
  protected $headers = [
    'Content-Type' => 'application/json',
    'Accept' => 'application/json',
  ];

  /**
   * Time Service.
   * 
   * @var \Drupal\Component\Datetime\TimeInterface
   */
  protected $time;

  /**
   * Acquia Subscription Service.
   *
   * @var \Drupal\acquia_connector\Subscription
   */
  protected $subscription;

  /**
   * AcquiaSearchApiClient constructor.
   *
   * @param \Drupal\Core\Logger\LoggerChannelInterface $logger_channel
   *   Logger Channel service.
   * @param \Drupal\acquia_connector\Subscription $subscription
   *   The Acquia subscription service.
   * @param \GuzzleHttp\Client $http_client
   *   HTTP client.
   * @param \Drupal\Core\Cache\CacheBackendInterface $cache
   *   Cache backend.
   */
  public function __construct(LoggerChannelInterface $logger_channel, Subscription $subscription, Client $http_client, CacheBackendInterface $cache,  TimeInterface $date_time) {
    $this->logger = $logger_channel;
    $this->subscription = $subscription;
    $this->client = $http_client;
    $this->time = $date_time;
    $this->cache = $cache;
  }

  /**
   * Helper function to fetch all search v3 indexes for given subscription.
   *
   * @return array|bool
   *   Acquia Search indexes array, FALSE on Acquia Search API failure.
   */
  public function getSearchIndexes() {
    $id = $this->subscription->getSettings()->getIdentifier();
    $cid = 'acquia_search.indexes.' . $id;
    $now = $this->time->getRequestTime();

    if (($cache = $this->cache->get($cid)) && $cache->expire > $now) {
      return $cache->data;
    }

    $path = '/v2/index/configure';
    $query_string = 'network_id=' . $id;
    $result = [];

    $indexes = $this->searchRequest($path, $query_string);
    if (empty($indexes) && !is_array($indexes)) {
      // When API is not reachable, cache it for 1 minute.
      $this->cache->set($cid, FALSE, $this->time->getRequestTime() + 60);

      return FALSE;
    }

    foreach ($indexes as $index) {
      $result[$index['key']] = [
        'balancer' => $index['host'],
        'core_id' => $index['key'],
        'data' => $index,
      ];
    }
    // Cache will be set in both cases, 1. when search v3 cores are found and
    // 2. when there are no search v3 cores but api is reachable.
    $this->cache->set($cid, $result, $now + (24 * 60 * 60));

    return $result;

  }

  /**
   * Create and send a request to search controller.
   *
   * @param string $path
   *   Path to call.
   * @param string $query_string
   *   Query string to call.
   *
   * @return array|false
   *   Response array or FALSE.
   */
  public function searchRequest(string $path, string $query_string) {
    $subscription_data = $this->subscription->getSubscription();
    // Return no results if there is no subscription data.
    if (!$subscription_data) {
      return false;
    }
    $host = $subscription_data['acquia_search']['api_host'];
    $req_time = $this->time->getRequestTime();
    $authorization_string = $this->calculateAuthString();
    $req_params = [
      'GET',
      preg_replace('/^https?:\/\//', '', $host),
      $path,
      $query_string,
      $authorization_string,
      $req_time,
    ];
    $authorization_header = $this->calculateAuthHeader($req_params, $authorization_string);

    $data = [
      'host' => $host,
      'headers' => [
        'Authorization' => $authorization_header,
        'X-Authorization-Timestamp' => $req_time,
      ],
    ];

    $uri = $data['host'] . $path . '?' . $query_string;
    $options = [
      'headers' => $data['headers'],
      'timeout' => 10,
    ];

    try {
      $response = $this->client->get($uri, $options);
      if (!$response) {
        throw new \Exception('Empty Response');
      }
      $stream_size = $response->getBody()->getSize();
      $data = Json::decode($response->getBody()->read($stream_size));
      $status_code = $response->getStatusCode();

      if ($status_code < 200 || $status_code > 299) {
        $this->logger->error("Couldn't connect to search v3 API: @message",
          ['@message' => $response->getReasonPhrase()]);
        return FALSE;
      }
      return $data;
    }
    catch (RequestException $e) {
      if ($e->getCode() == 401) {
        $this->logger->error("Couldn't connect to search v3 API:
          Received a 401 response from the API. @message", ['@message' => $e->getMessage()]);
      }
      elseif ($e->getCode() == 404) {
        $this->logger->error("Couldn't connect to search v3 API:
          Received a 404 response from the API. @message", ['@message' => $e->getMessage()]);
      }
      else {
        $this->logger->error("Couldn't connect to search v3 API:
          @message", ['@message' => $e->getMessage()]);
      }
    }
    catch (\Exception $e) {
      $this->logger->error("Couldn't connect to search v3 API: @message",
        ['@message' => $e->getMessage()]);
    }

    return FALSE;
  }

  /**
   * Creates an authenticator based on a HMAC V2 signer.
   *
   * @param array $req_params
   *   Request parameters.
   * @param string $authorization_string
   *   Authorization string.
   *
   * @return string
   *   Returns the signed auth header
   */
  private function calculateAuthHeader(array $req_params, string $authorization_string): string {
    $signature_base_string = implode("\n", $req_params);
    $digest = hash_hmac('sha256', $signature_base_string, base64_decode($this->subscription->getSettings()->getSecretKey(), TRUE), TRUE);
    $signature = base64_encode($digest);
    $authorization_header_string = str_replace("=", "=\"", str_replace("&", "\",", $authorization_string));
    return 'acquia-http-hmac ' . $authorization_header_string . '",signature="' . $signature . '"';
  }

  /**
   * Calculates authorization string.
   *
   * @return string
   *   Returns the auth string.
   */
  private function calculateAuthString() {

    $nonce = Crypt::randomBytesBase64(24);
    $uuid = $this->subscription->getSubscription()['uuid'];
    return 'id=' . $uuid . '&nonce=' . $nonce . '&realm=search&version=2.0';

  }

}
