<?php

namespace Drupal\acquia_search\Client\Solarium;

use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\acquia_search\Plugin\SolrConnector\SearchApiSolrAcquiaConnector;
use Drupal\search_api_solr\SolrConnectorInterface;
use Solarium\Core\Client\Endpoint as SolariumEndpoint;

/**
 * Custom Endpoint class for Solarium.
 *
 * This class takes custom Acquia Configuration for v3 endpoints.
 *
 * URL Pattern for SOLR 7+ QUERIES:
 *  "$SCHEME://$HOST:$PORT/$PATH/$CORE"
 *
 * @package Drupal\acquia_search
 */
class Endpoint extends SolariumEndpoint {

  const DEFAULT_NAME = 'search_api_solr';

  /**
   * Default name for Endpoint.
   *
   * @var string
   */
  protected $schema;

  /**
   * Options for putting together the endpoint urls.
   *
   * @var array
   */
  protected $options = [];

  /**
   * Class constructor.
   *
   * @param array $options
   *   Array of options for the endpoint. Currently,
   *   they are used by other functions of the endpoint.
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entityTypeManager
   *   Entity Type Manager service.
   *
   * @throws \Drupal\Component\Plugin\Exception\InvalidPluginDefinitionException
   * @throws \Drupal\Component\Plugin\Exception\PluginNotFoundException
   */
  public function __construct(array $options, EntityTypeManagerInterface $entityTypeManager) {
    /** @var \Drupal\search_api\ServerInterface $server */
    $server = $entityTypeManager->getStorage('search_api_server')->load(self::DEFAULT_NAME);
    $timeout_config = [];
    if ($server) {
      $connector_config = $server->getBackendConfig()['connector_config'];
      $timeout_config = [
        SolrConnectorInterface::QUERY_TIMEOUT => $connector_config['timeout'],
        SolrConnectorInterface::INDEX_TIMEOUT => $connector_config[SolrConnectorInterface::INDEX_TIMEOUT],
        SolrConnectorInterface::OPTIMIZE_TIMEOUT => $connector_config[SolrConnectorInterface::OPTIMIZE_TIMEOUT],
        SolrConnectorInterface::FINALIZE_TIMEOUT => $connector_config[SolrConnectorInterface::FINALIZE_TIMEOUT],
      ];
    }

    $options = array_merge(
      $timeout_config,
      SearchApiSolrAcquiaConnector::getPlatformConfig(),
      [
        'collection' => NULL,
        'leader' => FALSE,
      ],
      $options
    );

    parent::__construct($options);
  }

  /**
   * Fetch the Core Path, without the base uri.
   *
   * @return string
   *   Get the path and the core only w/o endpoint basepath.
   */
  public function getCorePath(): string {
    return vsprintf(
        '/%s/%s/',
        [
          $this->getPath(),
          $this->getCore(),
        ]
      );
  }
  /**
   * Get the V1 base url for all requests.
   *
   * @return string
   *   Get the base URI for the Endpoint plus path and the core vars.
   *
   * @throws \Solarium\Exception\UnexpectedValueException
   */
  public function getCoreBaseUri(): string {
    return $this->getBaseUri();
  }

  /**
   * Base URI for Endpoint
   *
   * @return string
   *   Base URL with scheme and port.
   */
  public function getBaseUri(): string {
    return vsprintf(
      '%s://%s:%d/%s/%s/',
      [
        $this->getScheme(),
        $this->getHost(),
        $this->getPort(),
        $this->getPath(),
        $this->getCore(),
      ]
    );
  }

  /**
   * Get the base url for all V1 API requests.
   *
   * @return string
   *   Base v1 URI for the endpoint.
   *
   * @throws \Solarium\Exception\UnexpectedValueException
   */
  public function getV1BaseUri(): string {
    return vsprintf(
      '%s://%s:%d/%s/',
      [
        $this->getScheme(),
        $this->getHost(),
        $this->getPort(),
        $this->getPath(),
      ]
    );
  }

  /**
   * Get the base url for all V2 API requests.
   *
   * @return string
   *   V2 base URI for the endpoint.
   *
   * @throws \Solarium\Exception\UnexpectedValueException
   */
  public function getV2BaseUri(): string {
    return $this->getBaseUri() . '/api/';
  }

  /**
   * Get the server uri, required for non core/collection specific requests.
   *
   * @return string
   *   Base URI for the endpoint.
   */
  public function getServerUri(): string {
    return $this->getBaseUri();
  }

  /**
   * Get the name of this endpoint.
   *
   * @return string|null
   *   Always use the default name.
   */
  public function getKey(): ?string {
    return self::DEFAULT_NAME;
  }

}
