<?php

namespace Drupal\acquia_search\EventSubscriber\PossibleCores;

use Drupal\acquia_connector\Subscription;
use Drupal\acquia_search\AcquiaSearchEvents;
use Drupal\acquia_search\Event\AcquiaPossibleCoresEvent;
use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Site\Settings;
use Drupal\Core\Messenger\MessengerInterface;
use Symfony\Component\EventDispatcher\EventSubscriberInterface;

/**
 * Gets the ContentHub Server settings from Drupal's settings.
 */
class LocalOverride implements EventSubscriberInterface {

  /**
   * Drupal Settings.
   *
   * @var \Drupal\Core\Site\Settings
   */
  protected $settings;

  /**
   * Acquia subscription service.
   *
   * @var \Drupal\acquia_connector\Subscription
   */
  protected $subscription;

  /**
   * Indicates if the local override is using deprecated settings.
   *
   * @var bool
   */
  protected $deprecated;

  /**
   * Messenger Service.
   *
   * @var \Drupal\pathauto\MessengerInterface
   */
  protected $messenger;

  /**
   * @var string
   */
  protected $acquia_search_config;

  /**
   * @var string
   */
  protected $acquia_search_solr_config;

  /**
   * Get Possible Cores from Cloud Constructor
   *
   * @param \Drupal\Core\Site\Settings $settings
   *   Core Settings
   */
  public function __construct(Settings $settings, Subscription $subscription, ConfigFactoryInterface $config_factory, MessengerInterface $messenger) {
    $this->settings = $settings;
    $this->deprecated = $config_factory->get('acquia_search.settings')->get('override_search_core') || $config_factory->get('acquia_search_solr.settings')->get('override_search_core');
    $this->acquia_search_config = $config_factory->get('acquia_search.settings')->get('override_search_core') ?? '';
    $this->acquia_search_solr_config = $config_factory->get('acquia_search_solr.settings')->get('override_search_core') ?? '';
    $this->messenger = $messenger;
    $this->subscription = $subscription;
  }

  /**
   * {@inheritdoc}
   */
  public static function getSubscribedEvents() {
    // Overridden fires first and supersedes all other possible cores.
    $events[AcquiaSearchEvents::GET_POSSIBLE_CORES][] = ['onGetPossibleCores', 9999];
    return $events;
  }

  /**
   * Gets a preset possible core from settings.
   *
   * @param \Drupal\acquia_search\Event\AcquiaPossibleCoresEvent $event
   *   The dispatched event.
   *
   * @see \Drupal\acquia_connector\Settings
   */
  public function onGetPossibleCores(AcquiaPossibleCoresEvent $event) {
    // Never allow overridden settings on Acquia Cloud
    if ($this->subscription->getProvider() === 'acquia_cloud') {
      return;
    }

    /**
     * Cover deprecated config.
     */
    if ($this->deprecated) {
      $this->messenger->addWarning("Acquia Search detected deprecated config settings. Please review README.txt and update 'override_search_core' setting to override.");
    }

    if ($this->acquia_search_config) {
      $core = $this->acquia_search_config;
      $event->addPossibleCore($core);
      $event->setReadOnly(FALSE);
      $event->stopPropagation();
    }

    if ($this->acquia_search_solr_config) {
      $core = $this->acquia_search_solr_config;
      $event->addPossibleCore($core);
      $event->setReadOnly(FALSE);
      $event->stopPropagation();
    }

    $search_settings = $this->settings::get('acquia_search');
    if (is_array($search_settings)) {
      $core = $search_settings['override_search_core'];
      $readonly = $search_settings['read_only'] ?? TRUE;
      if (isset($core)) {
        $event->addPossibleCore($core);
        $event->setReadOnly($readonly);
        $event->stopPropagation();
      }
    }
  }

}
